# SPDX-FileCopyrightText: 2025-present phdenzel <phdenzel@gmail.com>
#
# SPDX-License-Identifier: MIT
"""Regular expression for version string parsing."""

VERSION_RE = r'__version__\s*=\s*["\']([^"\']+)["\']'
